
GLOBAL_MK_NAME:=$(lastword $(MAKEFILE_LIST))
include $(dir $(GLOBAL_MK_NAME))/config.mk

.DEFAULT_GOAL := all
all: build
.PHONY: all

##################################################################
# -> stamp-download
.PHONY: download
download:
	for file in $(SRC_FILES) ; do         \
	    wget -N $(SRC_URL)/$$file ;       \
	done
	touch stamp-$@

stamp-download:
	$(MAKE) download
	touch $@

.PHONY: distclean
distclean:: clean
distclean::
	$(RM) -v $(SRC_FILES) stamp-download

##################################################################
# stamp-download -> stamp-extracted
.PHONY: extract
extract: stamp-extract
stamp-extracted: $(SRC_FILES) stamp-download
	for file in $(SRC_FILES) ; do         \
	    case $$file in                    \
	        *.tgz|*.tar.gz)               \
		    tar zvxf $$file ;;        \
	        *.zip)                        \
		    unzip -oj $$file ;;       \
	        *.bib|*.pdf|*.tex|*.bst|*.cls)\
	            ;;                        \
		*)                            \
		    echo "Unknown file type $$file"; exit 1 ;; \
	    esac                              \
	done
	$(if $(EXTRACT_TARGETS),$(MAKE) $(EXTRACT_TARGETS))
	touch $@

.PHONY: clean-extracted
clean-extracted: clean-patched
	$(RM) -v stamp-extracted
	for file in $(SRC_FILES) ; do         \
	    test -f $$file || continue ;      \
	    case $$file in                    \
	        *.tgz|*.tar.gz)               \
		    $(RM) -vr $$(tar tzf $$file | tac) ;; \
	        *.zip)                        \
		    $(RM) -v $$(unzip -lqq $$file | \
	sed -e 's/^ *[0-9]\+ \+[0-9-]\+ \+[0-9:]\+ \+\(.*[/]\)\?//') ;; \
	        *.bib|*.pdf|*.tex|*.bst|*.cls)\
	            ;;                        \
		*)                            \
		    echo "Unknown file type $$file"; exit 1 ;; \
	    esac ;                            \
	done

.PHONY: clean
clean:: clean-extracted

##################################################################
# stamp-extracted -> stamp-patched
stamp-patched: stamp-extracted
	if test -d patches ; then quilt push -a || test $$? = 2 ; fi
	touch $@

.PHONY: clean-patched
clean-patched:
	if test -d patches ; then quilt pop -a || test $$? = 2 ; fi
	$(RM) -vr .pc
	$(RM) -v stamp-patched

##################################################################
# stamp-patched -> stamp-prepare
stamp-prepare: stamp-patched
	touch $@
clean::
	$(RM) -v stamp-prepare

##################################################################
# stamp-prepare -> stamp-latex-ins
LATEX_INS_GENLIST_FILES=$(LATEX_INS_FILES:%=%.genlist)
ifneq ($(MODE),latex-ins)
stamp-latex-ins: stamp-prepare
	$(if $(LATEX_INS_FILES),$(MAKE) MODE=latex-ins stamp-latex-ins)
	touch $@
-include $(LATEX_INS_GENLIST_FILES)
else
include $(LATEX_INS_GENLIST_FILES)

stamp-latex-ins: $(LATEX_INS_FILES) $(LATEX_INS_GENLIST_FILES)

$(LATEX_INS_GENLIST_FILES): %.genlist: %
	latex --jobname=$* $*
	sed  -e 's,\\openout[0-9]* = `\(\./\)\?\(.*\)'"'.,$*_OUTPUTS += \2,p;d"  $*.log > $@

include $(LATEX_INS_GENLIST_FILES)
endif


clean::
	$(RM) -v stamp-latex-ins $(LATEX_INS_GENLIST_FILES) \
	    $(foreach file,$(LATEX_INS_FILES),$($(file)_OUTPUTS) $(file).log)

##################################################################
# stamp-latex-ins -> stamp-built
# build $(PS_BUILD) $(PDF_BUILD)
.PHONY: build $(PS_BUILD) $(PDF_BUILD)
build: stamp-built
stamp-built: stamp-latex-ins $(PS_BUILD) $(PDF_BUILD)
	$(if $(BUILD_TARGETS),$(MAKE) $(BUILD_TARGETS))
	touch $@
clean::
	$(RM) -v stamp-built
	$(if $(PS_BUILD),$(MAKE) -f ../LaTeX-dtx.mk distclean \
		LU_MASTERS="$(PS_BUILD:%.ps=%)" LU_FLAVORS=PS)
	$(if $(PDF_BUILD),$(MAKE) -f ../LaTeX-dtx.mk distclean \
		LU_MASTERS="$(PDF_BUILD:%.pdf=%)" LU_FLAVORS=PDF)
$(PS_BUILD): %.ps: stamp-prepare
	$(MAKE) -f ../LaTeX-dtx.mk LU_MASTERS="$*" LU_FLAVORS=PS $@
$(PDF_BUILD): %.pdf: stamp-prepare
	$(MAKE) -f ../LaTeX-dtx.mk LU_MASTERS="$*" LU_FLAVORS=PDF $@

##################################################################
# stamp-built -> 
.PHONY: install
install: stamp-built
	$(if $(INSTALL_TARGETS),$(MAKE) $(INSTALL_TARGETS))

define install-type # type prefixvar
INSTALL_TARGETS+=install-$(1)
install-$(1): $$(install_$(1))
	$$(if $$(install_$(1)),\
		install -d -m 755 $$(DESTDIR)$$($(2)$(1)dir))
	$$(foreach file,$$(install_$(1)),\
		install -m 644 $$(file) $$(DESTDIR)$$($(2)$(1)dir)/$$(notdir \
		$$(if $$($$(file)_RENAME),$$($$(file)_RENAME),$$(file))) && )\
	:
endef

README_RENAME ?= README.$(SUBPACKAGE)

$(foreach type,$(FILE_TYPES),\
	$(eval $(call install-type,$(type),pkg)))
$(eval $(call install-type,maindoc,))

##################################################################
# check
.PHONY: check
CHECK_PREFIX=@
check: stamp-download
	$(CHECK_PREFIX)$(RM) check-error
	$(CHECK_PREFIX)(                      \
	for file in $(SRC_FILES) ; do         \
	    test -f $$file || continue ;      \
	    case $$file in                    \
	        *.tgz|*.tar.gz)               \
		    tar tzf $$file ;; \
	        *.zip)                        \
		    unzip -lqq $$file | \
	sed -e 's/^ *[0-9]\+ \+[0-9-]\+ \+[0-9:]\+ \+\(.*[/]\)\?//' ;; \
	        *.bib|*.pdf|*.tex|*.bst|*.cls)\
		    echo "$$file" ;;          \
		*)                            \
		    echo "Unknown file type $$file" 1>&2 ; exit 1 ;; \
	    esac ;                            \
	done ;                                \
	for file in $(PS_BUILD) $(PDF_BUILD)  \
		$(foreach file,$(LATEX_INS_FILES),$($(file)_OUTPUTS)) ; do \
	    echo $$file ;                     \
	done                                  \
	) | while read file ; do              \
	    case " $(noinstall) $(foreach type,$(FILE_TYPES) maindoc,$(install_$(type)) )" in \
	        *\ $$file\ *) ;;              \
	        *) echo "*** I do not know what do do with '$$file'" 1>&2 ; \
		   touch check-error ;;                  \
	    esac ;                            \
	done
	$(CHECK_PREFIX)if test -f check-error ; then \
		$(RM) check-error ; \
		exit 1 ; \
	fi
	@echo "*** Congratulation: all files are correctly managed for $(SUBPACKAGE)"
